<?php
class forum_mmoo_data{
	var $wpdb;
	var $table;
	var $fields = array();
	var $needId, $needTime;
	var $default_rows = array();
	
	var $sqlToBeChecked = '';
	var $error = array();
	
	var $now = "DATE_FORMAT(now(),'%Y%m%d%H%i%s')";
	var $nonQuote;
	
	function __construct($name='', $needId=true, $needTime=true){
		global $wpdb;
		$this->wpdb = $wpdb;
		$this->setTable($name);
		$this->needId = $needId;
		$this->needTime = $needTime;
		
		if($needId){
			$this->setField( array('id' => array('col_name'=>'id', 'type'=>'int', 'length'=>11, 'index'=>'PRIMARY KEY', 'auto_increment'=>true)) );
		}
		if($needTime){
			$this->setField( array('updated_time' => array('col_name'=>'updated_time','type'=>'char', 'length'=>14, 'null'=>FORUM_MMOO_FLAG_NO) ));
		}
		
		$this->nonQuote = array($this->now);
	}
	
	function setTable($name){
		$strPath = dirname(dirname(__FILE__));
		$arrPath = explode('/', $strPath);
		$forum_mmoo = array_pop($arrPath);
		
		$this->table = $this->wpdb->prefix.$forum_mmoo.'_'.$name;
	}
	
	function setField($field){
		$this->fields = array_merge($this->fields, $field);
	}
	
	function setDefaultRows(){
	}
	
	function query($sql){
		if( FORUM_MMOO_DEBUG ){
			include_once(FORUM_MMOO_FUNCTIONS_DIR.'debug.php');
			forum_mmoo_debug_write('query', $sql);
		}
		$this->wpdb->query($sql);
	}
	
	function setError($error){
		$this->error[] = array($error);
		include_once(FORUM_MMOO_FUNCTIONS_DIR.'debug.php');
		forum_mmoo_debug_write('db_error', $error."\n".$this->sqlToBeChecked);
		$this->sqlToBeChecked = '';
	}
	
	function addQuote($col,$val){
		if( substr($col, 0, 9)=='SUBSTRING' || substr($col, 0, 9)=='substring' ){
			return array($col, " '".$val."' ");
		}
		
		$this->setAllFields();
		switch( $this->fields[$col]['type'] ){
			case 'char':
			case 'varchar':
			case 'text':
			case 'date':
			case 'time':
			case 'datetime':
				if( in_array($val,$this->nonQuote) ){
					return array($col, $val);
				}else{
					return array($col, " '".$val."' ");
				}
			default:
				return array($col, $val);
		}
	}
	
	function makeWhereOne($col, $val=''){
		if( empty($col) ){
			return '';
		}
		
		$where = '';
		
		if( is_array($val) ){
			$and = ' ';
			foreach($val as $operator => $val){
				if($operator=='not'){
					$operator = '<>';
				}
				switch($operator){
					case '=':
					case '<>':
					case '<':
					case '>':
					case '<=':
					case '>=':
						if( is_array($val) ){
							foreach($val as $v){
								$colAndVal = $this->addQuote($col,$v);
								$where = $and.$colAndVal[0].' '.$operator.' '.$colAndVal[1];
								$and = ' AND ';
							}
						}else{
							$colAndVal = $this->addQuote($col,$val);
							$where = $colAndVal[0].' '.$operator.' '.$colAndVal[1];
						}
					break;
					
					case 'in':
					case 'not in':
						$where = $col.' '.$operator.' ( ';
						foreach($val as $v){
							$colAndVal = $this->addQuote($col,$v);
							$where .= $and.$colAndVal[1];
							$and = ' , ';
						}
						$where .= ' ) ';
					break;
					
					case 'like':
						$where = $col." like '%".$val."%' ";
					break;
					
					default:
						$where = $col.$operator.$val;
					break;
				}
			}
		}else{
			$where = $this->makeWhereOne($col, array( '=' => $val ));
		}
		return $where;
	}
	
	function makeWhere($where, $first=null, $then=null){
		if( empty($where) ){
			return '';
		}
		
		$and = is_null($first) ? ' WHERE ' : $first;
		$and2 = is_null($then) ? ' AND ' : $then;
		
		$return = '';
		
		foreach( $where as $key => $val ){
			$return .= $and.' ( '.$this->makeWhereOne($key,$val).' ) ';
			$and = $and2;
		}
		return $return;
	}
	
	function mergeWhere($where, $then='AND'){
		$return = '';
		$and = '';
		foreach($where as $w){
			$return .= $and.' ( '.$w.' ) ';
			$and = $then;
		}
		return $return;
	}
	
	function executeSelect($selectCols='', $where='', $option=''){
		$sql = " SELECT ";
		if( empty($selectCols) ){
			$sql .= " * ";
		}elseif( !is_array($selectCols) ){
			$sql .= $selectCols;
		}else{
			$sql .= implode( ' , ' , $selectCols);
		}
		$sql .= " FROM ".$this->table;
		$sql .= $this->makeWhere($where);
		$sql .= $option;
		
		if( FORUM_MMOO_DEBUG ){
			include_once(FORUM_MMOO_FUNCTIONS_DIR.'debug.php');
			forum_mmoo_debug_write('select', $sql);
		}
		
		return $this->wpdb->get_results($sql, ARRAY_A);
	}
	
	function executeSelectOne($selectCols='', $where='', $option=''){
		$result = $this->executeSelect($selectCols, $where, $option);
		return $result[0];
	}
	
	function getCntRows($where=''){
		if( $this->needId ){
			$select = "COUNT(id) as cnt";
		}else{
			$select = "COUNT(*) as cnt";
		}
		$result = $this->executeSelectOne($select, $where);
		return $result['cnt'];
	}
	
	function makeLimitSql($perPage=null, $page=null){
		if( empty($perPage) ){
			return '';
		}
		
		if( empty($page) ){
			$page = 1;
		}
		$offset = $perPage * ( $page - 1 );
		return ' limit '.$offset.' , '.$perPage.' ';
	}
	
	function executeInsert($row){
		$before = $this->getCntRows();
		
		if( $this->needTime ){
			$row['updated_time'] = $this->now;
		}
		
		$arrSet = array();
		$arrVal = array();
		foreach($row as $col => $val){
			$colAndVal = $this->addQuote($col,$val);
			$arrSet[] = $colAndVal[0];
			$arrVal[] = $colAndVal[1];
		}
		
		$sql = " INSERT INTO ".$this->table." ( ".implode(' , ', $arrSet)." ) values ( ".implode(' , ', $arrVal)." ) ";
		$this->sqlToBeChecked = $sql;
		$this->query($sql);
		
		$after = $this->getCntRows();
		
		if( $after != $before + 1 ){
			$this->setError('failed to insert row to '.$this->table);
			return false;
		}
		return true;
	}
	
	function executeDelete($where=''){
		$rowNum = $this->getCntRows($where);
		if( $rowNum == 0 ){
			return true;
		}
		
		$before = $this->getCntRows();
		$sql = " DELETE FROM ".$this->table.$this->makeWhere($where);
		$this->query( $sql );
		$this->sqlToBeChecked = $sql;
		$after = $this->getCntRows();
		
		if( $after != $before - $rowNum ){
			$this->setError('failed to delete row from '.$this->table);
			return false;
		}
		return true;
	}
	
	function executeUpdate($set, $where, $insert=false){
		$rowNum = $this->getCntRows($where);
		if( $rowNum == 0 ){
			if( !$insert ){
				$this->setError('row to be updated doesnt exists in '.$this->table);
				return false;
			}else{
				return $this->executeInsert( array_merge($set,$where) );
			}
		}else{
			if( $this->needTime ){
				$set['updated_time'] = $this->now;
			}
			
			$sql = " UPDATE ".$this->table." SET ";
			$and = '';
			foreach($set as $col => $val){
				$colAndVal = $this->addQuote($col,$val);
				$sql .= $and.$colAndVal[0]." = ".$colAndVal[1];
				$and = " , ";
			}
			$sql .= $this->makeWhere($where);
			$this->query($sql);
			
			$this->sqlToBeChecked = $sql;
			unset($set['updated_time']);
			if( empty($where) ){
				$withoutSet = $this->getCntRows();
				$withSet = $this->getCntRows($set);
			}else{
				$withoutSet = $this->getCntRows($where);
				$withSet = $this->getCntRows(array_merge($where,$set));
			}
			
			if( $withoutSet != $withSet ){
				$this->setError('failed to update '.$this->table);
				return false;
			}
			return true;
		}
	}
	
	function makeFieldSql($field){
		$sql = " ".$field['col_name'];
		$sql .= " ".$field['type'];
		if( isset($field['length']) ){
			$sql .= "(".$field['length'].")";
		}
		if( isset($field['default']) ){
			$defaultVal = $this->addQuote($field['col_name'],$field['default']);
			$sql .= " DEFAULT ".$defaultVal[1];
		}
		if( $field['null']==FORUM_MMOO_FLAG_NO ){
			$sql .= " NOT NULL";
		}
		if( $field['auto_increment'] ){
			$sql .= " AUTO_INCREMENT";
		}
		if( isset($field['other']) ){
			$sql .= " ".$field['other'];
		}
		$sql .= " ";
		return $sql;
	}
	
	function executeCreate(){
		if ( ! empty($this->wpdb->charset) ){
			$charset_collate = " DEFAULT CHARACTER SET ".$this->wpdb->charset;
		}
		if ( ! empty($this->wpdb->collate) ){
			$charset_collate .= " COLLATE ".$this->wpdb->collate;
		}
		
		$sql = " CREATE TABLE IF NOT EXISTS ".$this->table." ( ";
		$and = '';
		$keys = array();
		
		$this->setAllFields();
		foreach($this->fields as $field){
			if( isset($field['index']) ){
				$keys[] = $field['index'].' ('.$field['col_name'].') ';
			}
			$sql .= $and;
			$and = ' , ';
			$sql .= $this->makeFieldSql($field);
		}
		if( !empty($keys) ){
			$sql .= ' , '.implode( ' , ', $keys);
		}
		$sql .= " ) ".$charset_collate;
		$this->query($sql);
		
		$result = $this->wpdb->get_results("show full columns from ".$this->table, ARRAY_A);
		if( empty($result) ){
			$this->error[] = 'failed to create table :: '.$this->table;
			return false;
		}
		return true;
	}
	
	function executeAlterAdd($field){
		$before = $this->wpdb->get_results("show full columns from ".$this->table, ARRAY_A);
		$this->query(" ALTER TABLE ".$this->table." ADD ".$this->makeFieldSql($field) );
		$after = $this->wpdb->get_results("show full columns from ".$this->table, ARRAY_A);
		if( count($after) != count($before) + 1 ){
			$this->error[] = 'failed to alter :: '.$this->table;
			return false;
		}
		return true;
	}
	
	function newTable(){
		$this->setDefaultRows();
		$old = $this->wpdb->get_results("show full columns from ".$this->table, ARRAY_A);
		
		if( empty($old) ){
			if( !$this->executeCreate() ){
				return false;
			}
			$insertSuccess = true;
			if( !empty($this->default_rows) ){
				foreach( $this->default_rows as $row ){
					if( !$this->executeInsert($row) ){
						$insertSuccess = false;
					}
				}
			}
			return $insertSuccess;
		}else{
			$this->setAllFields();
			$fieldToAdd = $this->fields;
			foreach($old as $already){
				unset( $fieldToAdd[$already['Field']] );
			}
			
			if( empty($fieldToAdd) ){
				return true;
			}else{
				foreach($fieldToAdd as $fieldName => $fieldInfo){
					if( !$this->executeAlterAdd($fieldInfo) ){
						return false;
					}
					if( isset($fieldInfo['default']) ){
						if( !$this->executeUpdate( array($fieldName=>$fieldInfo['default']), '') ){
							return false;
						}
					}
				}
				return true;
			}
		}
	}
	
	function dropTable(){
		$this->query(" DROP TABLE ".$this->table );
		$after = $this->wpdb->get_results("show full columns from ".$this->table, ARRAY_A);
		if( !empty($after) ){
			$this->error[] = 'failed to drop :: '.$this->table;
			return false;
		}
		return true;
	}
	
	function decodeFlag($colName, $memberParamName, $row){
		foreach($this->{$memberParamName} as $keyName => $info){
			if( $row[$colName] == $info['flag'] ){
				$row[$colName] = $keyName;
				break;
			}
		}
		return $row;
	}
	
	function encodeFlag($colName, $memberParamName, $row){
		$row[$colName] = $this->{$memberParamName}[$row[$colName]]['flag'];
		return $row;
	}
	
	function getNow(){
		$result = $this->executeSelectOne(array($this->now.' as n'));
		return $result['n'];
	}
}
?>
