<?php
include_once(FORUM_MMOO_DATA_DIR.'_data.php');

class forum_mmoo_data_thread extends forum_mmoo_data{
	function __construct($topicId){
		parent::__construct('thread_'.$topicId);
	}
	
	function setAllFields(){
		$fields = array(
			'title' => array('col_name'=>'title', 'type'=>'varchar', 'length'=>256)
			, 'posted_by' => array('col_name'=>'posted_by', 'type'=>'int', 'length'=>11, 'index'=>' INDEX posted_by ')
			, 'posted_date' => array('col_name'=>'posted_date', 'type'=>'char', 'length'=>14)
			, 'content' => array('col_name'=>'content', 'type'=>'text', 'index'=>' FULLTEXT content ')
			, 'direct_parent' => array('col_name'=>'direct_parent', 'type'=>'int', 'length'=>11)
			, 'first_parent' => array('col_name'=>'first_parent', 'type'=>'int', 'length'=>11, 'index'=>' INDEX first_parent ')
			, 'depth' => array('col_name'=>'depth', 'type'=>'int', 'length'=>4, 'default'=>0 )
		);
		$this->setField($fields);
	}
	
	function getPostNum( $cntReply=true, $searchWords='' ){
		include_once(FORUM_MMOO_FUNCTIONS_DIR.'search_string.php');
		
		if( $cntReply ){
			$select = 'count(id) as cnt';
		}else{
			$select = 'count( distinct ifnull(first_parent,id) ) as cnt';
		}
		
		$searchWhereContent = forum_mmoo_get_search_string_where('content', forum_mmoo_search_string_into_array($searchWords));
		$searchWhereTitle = forum_mmoo_get_search_string_where('title', forum_mmoo_search_string_into_array($searchWords));
		$option = empty($searchWhereContent) ? '' : ' WHERE '.$searchWhereContent.' OR '.$searchWhereTitle;
		$result = $this->executeSelectOne(array($select), '', $option);
		return $result['cnt'];
	}
	
	function getData( $perPage=null, $page=1, $sort='desc', $searchWords='' ){
		include_once(FORUM_MMOO_FUNCTIONS_DIR.'search_string.php');
		$return = array();
		
		$searchWhereContent = forum_mmoo_get_search_string_where('content', forum_mmoo_search_string_into_array($searchWords));
		$searchWhereTitle = forum_mmoo_get_search_string_where('title', forum_mmoo_search_string_into_array($searchWords));
		if( empty($searchWhereContent) ){
			$where = array('depth'=>0);
			$optionalWhere = '';
		}else{
			$where = '';
			$optionalWhere = ' WHERE '.$searchWhereContent.' OR '.$searchWhereTitle;
		}
		
		$orderBy = ' ORDER BY id '.$sort;
		
		$limit = $this->makeLimitSql($perPage, $page);
		
		$firstParents = $this->executeSelect( ' distinct ifnull(first_parent,id) as first ', $where, $optionalWhere.$orderBy.$limit );
		if( empty($firstParents) ){
			return array();
		}
		
		foreach( $firstParents as $parent ){
			$firstParentId[] = $parent['first'];
		}
		
		$whereChildren = ' WHERE ';
		$whereChildren .= ' id in ( '.implode(' , ',$firstParentId).' ) ';
		$whereChildren .= ' OR ';
		$whereChildren .= ' first_parent in ( '.implode(' , ',$firstParentId).' ) ';
		
		$children = $this->executeSelect( '', '', $whereChildren);
		
		foreach( $children as $childRow ){
			$return[$childRow['id']] = $childRow;
		}
		return $return;
	}
	
	function getPageForPost($postId, $perPage, $sort='desc'){
		$firstParentResult = $this->executeSelectOne( array(' ifnull(first_parent,id) as fff ') , array('id'=>$postId) );
		$firstParent = $firstParentResult['fff'];
		
		$operator = ( $sort=='asc' ) ? '<=' : '>=';
		$where = array(
			'id' => array($operator => $firstParent)
			, 'depth' => 0
		);
		$ahead = $this->getCntRows($where);
		return ceil( $ahead / $perPage) ;
	}
	
	function getPost($id){
		return $this->executeSelectOne('', array('id'=>$id) );
	}
	
	function getPosts($arrId){
		$where = array( 
			'id' => array(
				'in' => $arrId
			)
		);
		return $this->executeSelect( '', $where );
	}
	
	function deletePost($id){
		$childNum = $this->getCntRows(array('first_parent'=>$id));
		if( $childNum > 0 ){
			return false;
		}
		return $this->executeDelete( array('id'=>$id) );
	}
	
	function getLastUpdate(){
		return $this->executeSelectOne('','',' WHERE id = ( SELECT max(id) FROM '.$this->table.' ) ');
	}
}
?>
